'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
import os

class Config:

    @classmethod
    def tushare_token(cls):
        return os.environ.get('TUSHARE_TOKEN') or 'fb8f2489e74b45afc06d859d9a45ea48f424e6c87d971b0af724b261'

    @classmethod
    def heartbeat_timelength(cls):
        return int(os.environ.get('HEARTBEAT_TIMELENGTH') or '5') 

    @classmethod
    def bars_length(cls):
        return int(os.environ.get('BARS_LENGTH') or '45')

    @classmethod
    def bars_length_1m(cls):
        return int(os.environ.get('BARS_LENGTH_1M') or '2880')
    
    @classmethod
    def heartbeat_length_m(cls):
        return int(os.environ.get('HEARTBEAT_LENGTH_1M') or '300')
    
    @classmethod
    def bars_length_5m(cls):
        return int(os.environ.get('BARS_LENGTH_5M') or '576')
    

    @classmethod
    def benchmarks(cls):
        return (os.environ.get('BENCHMARK_VARIETIES') or 'csi5,csi6-20,csi21-100,csi100').split(',')

    @classmethod
    def mode(cls):
        return os.environ.get('MODE') or 'local'  # 'jupyter'




'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
import datetime
import random

from vitu.report.report import Report
from vitu.account.account_manager import AccountManager
from vitu.utils.date_utils import timestamp2str,str2datetime

class CompleteReport(Report):
    def __init__(self, portfolio):
        super(CompleteReport, self).__init__(portfolio)
        self.completed_time = None
        self.benchmark_annualized_volatility = None

        self.orders = None
        self.portfolio_positions = None
        self.config = None
        self.code = None
        self.log = None

    def get_dates_values_positions_orders(self):
        """
        :return: dates、values、positions、orders
        """
        values = list()
        dates = list()
        portfolio_position = dict()
        orders = dict()
        benchmarks=self.benchmark 
        values.append(self.context.init_position_total)
        for date, pf in self.rebalance_history.items():
            values.append(sum([asset['total_amount'] for asset in pf['portfolio_position']['detail'].values()]))
            dates.append(date)
            portfolio_position[date] = pf['portfolio_position']
            orders[date] = pf['orders']
        date1=[]
        value1=[]
        portfolio_position1=dict()
        orders1 = dict()
        freq1=self.context.frequency
        if freq1 in ['d','1d','day','1day']:
            for i, v in enumerate(dates):
                if v in benchmarks['timestamp'].values:
                   date1.append(dates[i])
                   value1.append(values[i])
                   portfolio_position1[dates[i]]= portfolio_position[dates[i]]
                   orders1[dates[i]] = orders[dates[i]] 
        else:
            for i, v in enumerate(dates):
                if v in benchmarks['timestamp'].values:
                    countbar=0
                    maxnum=random.uniform(5,10)
                    date1.append(dates[i])
                    value1.append(values[i])
                    portfolio_position1[dates[i]]= portfolio_position[dates[i]]
                if  orders[dates[i]]: 
                    currentdate=str2datetime(dates[i]).replace(hour=0, minute=0, second=0, microsecond=0)
                    if bool(orders1):
                       if (str(currentdate)) not in orders1.keys():
                          orders1[str(currentdate)]=[]
                       tempdate=list(orders1.keys())[-1]
                       orderdate=str2datetime(tempdate).replace(hour=0, minute=0, second=0, microsecond=0)
                       if currentdate==orderdate:
                          countbar+=1
                          if countbar<=maxnum: 
                            orders1[str(currentdate)].append(orders[dates[i]][0])
                       else: 
                            orders1[str(currentdate)].append(orders[dates[i]][0])
                    else:
                        currentdate=str2datetime(dates[i]).replace(hour=0, minute=0, second=0, microsecond=0) 
                        orders1[str(currentdate)]=[]
                        orders1[str(currentdate)].append(orders[dates[i]][0])
        dates=date1
        values=value1
        portfolio_position=portfolio_position1
        orders=orders1
        return dates,values,portfolio_position,orders

    def get_cross_months(self, start_date, end_date):
        """
        :param start_date: "2019-05-31"
        :param end_date: "2019-08-03"
        :return: months 两个日期间跨几个月
        """
        start_year = datetime.datetime.strptime(start_date[:10], "%Y-%m-%d").year
        end_year = datetime.datetime.strptime(end_date[:10], "%Y-%m-%d").year
        start_month = datetime.datetime.strptime(start_date[:10], "%Y-%m-%d").month
        end_month = datetime.datetime.strptime(end_date[:10], "%Y-%m-%d").month
        months = (end_year - start_year) * 12 + (end_month - start_month + 1)
        return months

    def risk_metrics(self, st_returns, bm_returns):
        """
        :param st_returns: 策略相对收益率 列表
        :param bm_returns: 基准相对收益率 列表
        :return: 计算指标（核心）
        """
        cumulative_returns = self.get_cumulative_returns(st_returns)[0]
        benchmark_cumulative_returns = self.get_cumulative_returns(bm_returns)[0]

        annualized_return = self.get_annualized_return(cumulative_returns)
        benchmark_annualized_return = self.get_annualized_return(benchmark_cumulative_returns)
        max_drawdown = self.get_max_drawdown(cumulative_returns)

        annualized_volatility = self.get_annualized_volatility(st_returns)
        benchmark_annualized_volatility = self.get_annualized_volatility(bm_returns)
        rf = self.get_riskfree_rate()
        alpha, beta = self.get_CAPM(st_returns, bm_returns, rf)
        sharpe = self.get_sharpe(st_returns, rf)
        information_ratio = self.get_information_ratio(st_returns, bm_returns)

        report = {
            "annualized_return": annualized_return,
            "benchmark_annualized_return": benchmark_annualized_return,
            "alpha": alpha,
            "sharpe": sharpe,
            "information_ratio": information_ratio,
            "winning_ratio": None,
            "beta": beta,
            "annualized_volatility": annualized_volatility,
            "benchmark_annualized_volatility": benchmark_annualized_volatility,
            "max_drawdown": max_drawdown
        }
        return report

    def get_monthly(self, dates, number):
        """
        :param dates: 日期
        :param number: 几个月
        :return: 获取n个月字符串 列表
        """
        start_year = datetime.datetime.strptime(dates[0], "%Y-%m-%d %H:%M:%S").year
        start_month = datetime.datetime.strptime(dates[0], "%Y-%m-%d %H:%M:%S").month
        end_year = datetime.datetime.strptime(dates[-1], "%Y-%m-%d %H:%M:%S").year
        end_month = datetime.datetime.strptime(dates[-1], "%Y-%m-%d %H:%M:%S").month
        start_month += (number-1)
        if start_month > 12:
            start_month = start_month -12
            start_year += 1

        monthly = []
        while True:
            if start_year < end_year:
                monthly.append([start_year, start_month])
                start_month += 1
                if start_month > 12:
                    start_month = 1
                    start_year += 1
            if start_year == end_year:
                monthly.append([start_year, start_month])
                start_month += 1
                if start_month > end_month:
                    break
        return monthly

    def get_indexs(self, dates):
        """
        :param dates: 日期
        :return: 按1个月划分索引切点 列表
        """
        one_monthly = self.get_monthly(dates, 1)
        indexs = list()
        for monthly_index, monthly in enumerate(one_monthly):
            for date_index, date in enumerate(dates):
                if monthly_index == 0 and date_index == len(dates) - 1:
                    indexs.append(date_index)
                day = datetime.datetime.strptime(date, "%Y-%m-%d %H:%M:%S")
                if day.year == monthly[0] and day.month == monthly[1]:
                    indexs.append(date_index)
                    break  # 匹配到，就跳出内循环
        indexs.append(len(dates) - 1)  # 增加最后索引
        indexs.sort()
        return indexs

    def calculate_monthly_metrics(self, month, dates, st_returns, bm_returns):
        """
        :param month: 几个月
        :param dates: 日期
        :param st_returns: 策略相对收益率 列表
        :param bm_returns: 基准相对收益率 列表
        :return: 计算几个月的指标
        """
        monthly = self.get_monthly(dates,month)
        indexs = self.get_indexs(dates)
        report = dict()
        for num in range(len(monthly)):
            date_format = str()
            if len(str(monthly[num][1])) == 1:
                date_format = "{}-0{}".format(monthly[num][0],monthly[num][1])
            if len(str(monthly[num][1])) == 2:
                date_format = "{}-{}".format(monthly[num][0],monthly[num][1])
            left = indexs[num]
            right = indexs[num+month]
            if left>0 :
               left-=1
               right-=1
               
            if right == left:
                if right==len(st_returns):
                    left-=2
                    right-=1
                else:
                    left-=1 
            report[date_format] = self.risk_metrics(st_returns[left:right], bm_returns[left:right])

        new_report = {
            "annualized_return": {},
            "benchmark_annualized_return": {},
            "alpha": {},
            "sharpe": {},
            "information_ratio": {},
            "winning_ratio": {},
            "beta": {},
            "annualized_volatility": {},
            "benchmark_annualized_volatility": {},
            "max_drawdown": {},
        }
        for date, metrics in report.items():
            new_report["annualized_return"][date] = metrics["annualized_return"]
            new_report["benchmark_annualized_return"][date] = metrics["benchmark_annualized_return"]
            new_report["alpha"][date] = metrics["alpha"]
            new_report["sharpe"][date] = metrics["sharpe"]
            new_report["information_ratio"][date] = metrics["alpha"]
            new_report["winning_ratio"][date] = metrics["sharpe"]
            new_report["beta"][date] = metrics["beta"]
            new_report["annualized_volatility"][date] = metrics["annualized_volatility"]
            new_report["benchmark_annualized_volatility"][date] = metrics["benchmark_annualized_volatility"]
            new_report["max_drawdown"][date] = metrics["max_drawdown"]
        return new_report

    def run(self):
        import time
        # start1 = time.time()

        self.dates,self.values,self.portfolio_positions,self.orders = self.get_dates_values_positions_orders()
        self.bm_values = self.get_benchmark_values(self.dates)
        st_returns = self.get_relative_returns(self.values)
        bm_returns = self.get_relative_returns(self.bm_values)

        report = dict()
        report["last_one_month"] = self.calculate_monthly_metrics(1, self.dates, st_returns, bm_returns)
        cross_months = self.get_cross_months(self.context.portfolio.strategy.start, self.context.portfolio.strategy.end)
        if cross_months >= 3:
            report["last_three_month"] = self.calculate_monthly_metrics(3, self.dates, st_returns, bm_returns)
        if cross_months >= 6:
            report["last_six_month"] = self.calculate_monthly_metrics(6, self.dates, st_returns, bm_returns)
        if cross_months >= 12:
            report["last_twelve_month"] = self.calculate_monthly_metrics(12, self.dates, st_returns, bm_returns)

        new_report = {
            "display_type": "strategy_detail",
            # 风险指标
            "returns":{},
            "alpha": {},
            "sharpe": {},
            "information_ratio": {},
            "winning_ratio": {},
            "beta": {},
            "volatility":{},
            "max_drawdown": {},
        }
        for date, metrics in report.items():
            new_report["returns"][date] = {'annualized_return': metrics['annualized_return'],
                          'benchmark_annualized_return': metrics['benchmark_annualized_return']}
            excess_return = {}
            for key, value in metrics['annualized_return'].items():
                excess_return[key] = round((value - metrics['benchmark_annualized_return'][key]), 4)
            new_report["returns"][date]['excess_return'] = excess_return

            new_report["alpha"][date] = {"alpha": metrics["alpha"]}
            new_report["sharpe"][date] = {"sharpe": metrics["sharpe"]}
            new_report["information_ratio"][date] = {"information_ratio": metrics["information_ratio"]}
            new_report["beta"][date] = {"beta": metrics["beta"]}

            new_report["volatility"][date] = {'annualized_volatility': metrics['annualized_volatility'],
                          'benchmark_annualized_volatility': metrics['benchmark_annualized_volatility']}
            excess_volatility = {}
            for key, value in metrics['annualized_volatility'].items():
                excess_volatility[key] = round((value - metrics['benchmark_annualized_volatility'][key]), 4)
            new_report["volatility"][date]['excess_volatility'] = excess_volatility

            new_report["max_drawdown"][date] = {"max_drawdown": metrics["max_drawdown"]}

        orders_list = []
        for _,orders in self.orders.items():
            orders_list.extend(orders)

        filled_price = []
        filled_qty = []
        filled_amount = []
        commission = []
        if not orders_list:
            filled_price = [0]
            filled_qty = [0]
            filled_amount = [0]
            commission = [0]
        else:
            for order in orders_list:
                filled_price.append(order['limit_price'])
                filled_qty.append(order['filled_qty'])
                filled_amount.append(order['filled_amount'])
                commission.append(order['commission'])

        positions_list = []
        for _,positions in self.portfolio_positions.items():
            positions_list.extend(positions['detail'].values())
        total_qty = []
        total_amount = []
        value = []
        last_profit_and_loss = []
        percentage_of_positions = []
        avg_cost_btc = []
        avg_cost_usdt = []
        for position in positions_list:
            total_qty.append(position['total_qty'])
            total_amount.append(position['total_amount'])
            value.append(position['value'])
            last_profit_and_loss.append(position['last_profit_and_loss'])
            percentage_of_positions.append(position['percentage_of_positions'])
            avg_cost_btc.append(position['avg_cost_btc'])
            avg_cost_usdt.append(position['avg_cost_usdt'])

        new_report["info"] = {"orders":{"account_name":self.context.accounts_name,
                                        "universe": self.context.portfolio.strategy.universe,
                                        "filled_price":{'max':max(filled_price),'min':min(filled_price)},
                                        "filled_qty":{'max':max(filled_qty),'min':min(filled_qty)},
                                        "filled_amount":{'max':max(filled_amount),'min':min(filled_amount)},
                                        "commission":{'max':max(commission),'min':min(commission)}
                                        },
                              "positions":{"cryptos":self.context.asset_varieties,
                                           "total_qty":{'max':max(total_qty),'min':min(total_qty)},
                                           "total_amount": {'max':max(total_amount),'min':min(total_amount)},
                                           "value":{'max':max(value),'min':min(value)},
                                           "last_profit_and_loss":{'max':max(last_profit_and_loss),'min':min(last_profit_and_loss)},
                                           "percentage_of_positions":{'max':max(percentage_of_positions),'min':min(percentage_of_positions)},
                                           "avg_cost_btc":{'max':max(avg_cost_btc),'min':min(avg_cost_btc)},
                                           "avg_cost_usdt": {'max': max(avg_cost_usdt),'min': min(avg_cost_usdt)},
                                           }}

        new_report["completed_time"] = self.context.completed_time

        # 过滤当天为空的订单
        orders = {date:order for date,order in self.orders.items() if order}
        new_report["orders"] = orders

        new_report["portfolio_positions"] = self.portfolio_positions

        # time.sleep(0.2)
        run_end = self.context.clock.run_end if self.context.clock.run_end else time.time()
        config = {"backtest_interval":{"start":self.context.portfolio.strategy.start,
                                       "end":self.context.portfolio.strategy.end},
                  "account_name":[account.detail() for account in AccountManager().accounts_info],
                  "freq":self.context.portfolio.strategy.frequency,
                  "run_time":{"start":timestamp2str(self.context.clock.run_start),
                              "end":timestamp2str(run_end)}}
        new_report["config"] = config

        # for name in account_manager.accounts_name:
        #     name_position = "{}_position".format(name)
        #     new_report[name_position] = [{date:pf[name_position]}for date, pf in self.rebalance_history.items()]

        # start22 = time.time()
        for name in self.context.accounts_name:
            name_position = "{}_position".format(name)
            new_report[name_position] = dict()
            account_first = True

            for date, portfolio in self.rebalance_history.items():
                if date in self.benchmark['timestamp'].values:
                    last_date = str(datetime.datetime.strptime(date, '%Y-%m-%d %H:%M:%S') - datetime.timedelta(days=1))
                    asset_value = dict()
                    # 计算每个account的各个asset的last_profit_and_loss
                    for asset in portfolio["portfolio_position"]['detail'].values():
                        if name in asset["consist_of"].keys():
                            asset_value[asset["asset"]] = asset["consist_of"][name]
                            # if account_first:
                            #     last_profit_and_loss = (asset["consist_of"][name]['amount'] / self.context.init_portfolio_position[asset["asset"]]["consist_of"][name]['amount']) - 1
                            #     asset_value[asset["asset"]]['last_profit_and_loss'] = round(last_profit_and_loss, 4)
                            if account_first:
                                asset_value[asset["asset"]]['last_profit_and_loss'] = 0
                            else:
                                try:
                                    last_profit_and_loss = (asset["consist_of"][name]['amount'] / self.rebalance_history[last_date]["portfolio_position"]['detail'][asset["asset"]]["consist_of"][name]['amount']) - 1
                                    asset_value[asset["asset"]]['last_profit_and_loss'] = round(last_profit_and_loss, 4)
                                except:
                                    asset_value[asset["asset"]]['last_profit_and_loss'] = 0


                            # 买入数量、买入均价、卖出数量、卖出均价
                            # for d, orders in self.orders.items():
                            #     if date == d:
                            #         buy_avg_price = []
                            #         buy_qty = []
                            #         sell_avg_price = []
                            #         sell_qty = []
                            #         for order in orders:
                            #             if order['instrument'].split('/')[0].split('/')[0] == asset["asset"]:
                            #                 if order['side'] == 'buy':
                            #                     buy_avg_price.append(order['avg_price'])
                            #                     buy_qty.append(order['filled_qty'])
                            #                 elif order['side'] == 'buy':
                            #                     sell_avg_price.append(order['avg_price'])
                            #                     sell_qty.append(order['filled_qty'])
                            #         buy = {'buy_avg_price': np.mean(buy_avg_price) if buy_avg_price else 0,
                            #                'buy_qty': np.sum(buy_qty) if buy_qty else 0}
                            #         sell = {'sell_avg_price': np.mean(sell_avg_price) if sell_avg_price else 0,
                            #                 'sell_qty': np.sum(sell_qty) if sell_qty else 0}
                            #         asset_value[asset["asset"]]['buy'] = buy
                            #         asset_value[asset["asset"]]['sell'] = sell

                    account_first = False
                    position = {"detail": dict(), "total": dict()}
                    position['detail'] = asset_value
                    new_report[name_position][date] = position

                    # 计算amount
                    amount = 0
                    for asset,value in asset_value.items():
                        amount += value['amount']
                    total = {"assets":list(asset_value.keys()),
                            "amount":round(amount,4)}
                    new_report[name_position][date]['total'] = total

                    # 计算所占百分比
                    for asset, value in asset_value.items():
                        value['percentage_of_positions'] = round(value['amount']/new_report[name_position][date]['total']['amount'], 4)

                # 计算每个account的总的last_profit_and_loss
                date_first = True
                for date,value in new_report[name_position].items():
                    last_date = str(datetime.datetime.strptime(date, '%Y-%m-%d %H:%M:%S') - datetime.timedelta(days=1))
                    if date_first:
                        last_profit_and_loss = (value['total']['amount']/self.context.init_total_account_position[name_position])-1
                    else:
                        last_profit_and_loss = (value['total']['amount']/new_report[name_position][last_date]['total'])-1
                    value['total']['last_profit_and_loss'] = round(last_profit_and_loss, 4)

            # for name in self.context.accounts_name:
            #     name_position = "{}_position".format(name)
            #     new_report[name_position] = dict()
            #     for date, pf in self.rebalance_history.items():
            #         asset_value = dict()
            #         assets = list(pf["portfolio_position"].values())
            #         assets.pop()
            #         for asset in assets:
            #             if name in asset["consist_of"].keys():
            #                 asset_value[asset["asset"]] = asset["consist_of"][name]
            #         new_report[name_position][date] = asset_value
            # print("【name_position耗时】：{}".format(time.time()-start22))





        # print("【complete_report耗时】：{} s".format(str(time.time() - start1)[:5]))

        AccountManager().accounts_info = []
        return new_report



if __name__ == "__main__":
    pass








'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
#!/usr/bin/env python
#coding:utf-8


class UserError(Exception):
    # 用户代码导致的相关异常
    pass


class InternalError(Exception):
    # 系统内部的异常
    pass


class ParamsError(UserError):
    # 用户传入的api参数不合理
    pass


class SymbolNotSupported(UserError):
    # 用户尝试使用标的不支持
    pass


class MissDataError(InternalError):
    # 数据缺失导致的异常
    pass


class InitObjError(InternalError):
    # 常用于初始化对象时由于内部错误（比如数据错误）抛出的异常
    pass


class InvalidDataError(InternalError):
    # 数据不正常抛出的异常
    pass


class TimeoutError(InternalError):
    # 超时时触发的异常
    pass



'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
# from vitu import API, Config
from vitu.trade.position.spot_position import SpotPosition
from vitu.utils.data_utils import get_btc_usdt_cost


def order_update_position(context, current_position, side, exchange, symbol, price, qty):
    """
    产生Order更新对应Postion
    :param side: buy/sell
    :param exchange: binance
    :param symbol: BTC/USDT
    :param price: 价格
    :param qty: 数量
    :return:
    """
    base_currency = symbol.split('/')[0].lower()   # btc  基础货币
    quote_currency = symbol.split('/')[1].lower()  # usdt 计价货币
    if base_currency not in context.asset_varieties:
        context.asset_varieties.append(base_currency)
    if quote_currency not in context.asset_varieties:
        context.asset_varieties.append(quote_currency)

    if base_currency not in current_position.keys():
        # 计算初始持仓成本
        avg_cost_btc, avg_cost_usdt = get_btc_usdt_cost(base_currency, context.current_datetime())
        position = SpotPosition('spot', base_currency, avg_cost_btc=avg_cost_btc, avg_cost_usdt=avg_cost_usdt)
        position.context = context
        current_position[base_currency] = position

    if quote_currency not in current_position.keys():
        # 计算初始持仓成本
        avg_cost_btc, avg_cost_usdt = get_btc_usdt_cost(quote_currency, context.current_datetime())
        position = SpotPosition('spot', quote_currency, avg_cost_btc=avg_cost_btc, avg_cost_usdt=avg_cost_usdt)
        position.context = context
        current_position[quote_currency] = position
    current_position[base_currency].order_update('base', side, price, qty)
    current_position[quote_currency].order_update('quote', side, price, qty)

def trade_update_position(current_position, symbol, trade):
    """
    产生Trade更新对应Postion
    """
    base_currency = symbol.split('/')[0].lower()   # btc  基础货币
    quote_currency = symbol.split('/')[1].lower()  # usdt 计价货币
    current_position[base_currency].trade_update('base', trade)
    current_position[quote_currency].trade_update('quote', trade)
    # 更新open_cost开仓成本
    current_position[base_currency].trade_update_cost('base', trade, current_position[quote_currency])
    current_position[quote_currency].trade_update_cost('quote', trade, current_position[base_currency])





'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
class Position(object):
    __slots__ = [
        'asset_class',
        'asset',
    ]
    def __init__(self, asset_class, asset):
        self.asset_class = asset_class
        self.asset = asset

    def __getitem__(self, key, default=None):
        item_value = self.__getattribute__(key) if self.__getattribute__(key) else default
        return item_value




















'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
import pandas as pd
import numpy as np
import time
from datetime import datetime

from vitu.utils.date_utils import get_now_time, get_total_timestamps,get_total_dates
from vitudata.apis import get_bars

def get_data(exchange, symbol, frequency, start_date, end_date,timezone):
    data = get_bars(symbol, exchange, frequency=frequency, start_date=start_date, end_date=end_date,timezone=timezone)
    df_data = pd.DataFrame(data)
    # total_ts = get_total_timestamps(start_date, end_date)     #只能获得天的日期
    total_ts =get_total_dates(frequency,1,0,start_date, end_date)  #天或者分钟的日期，
    #对df_data清醒数据清洗
    if len(df_data)>1:
        if df_data['timestamp'][0]>0 and df_data['close'][0]>0:
           for x in range(1,len(df_data)):
               if df_data['close'][x]==0:
                  df_data.iloc[x,1:6]=df_data.iloc[x-1,1:6]

        else:
            nonzero=df_data[df_data['close']>0].index.tolist()
            if len(nonzero)>0:
                for n in range(0,nonzero[0]): 
                   df_data.iloc[n,1:6]=df_data.iloc[nonzero[0],1:6]
                for x in range(1,len(df_data)):
                    if df_data['high'][x]==0 and df_data['close'][x]==0:
                       df_data.iloc[x,1:6]=df_data.iloc[x-1,1:6]
     
    if len(df_data)==len(total_ts):
        df_data['timestamp'] = total_ts
    else:
        print('data error in cleaning, please wait')
        df_date=np.array(df_data['timestamp'] ,dtype='uint64')
        ts_date=[]
        for i in range(0,len(total_ts)):
            temp1=datetime.strptime(total_ts[i],'%Y-%m-%d %H:%M:%S').timestamp()
            ts_date.append(round(temp1)) #total_ts转化为时间戳形式 
        ts_date=np.array(ts_date,dtype='uint64')
        temp2=np.zeros([len(ts_date),5])
        df_data1=pd.DataFrame(temp2,index=range(len(ts_date)),columns=['open','high','low','close','volume'])    
        df_data1.insert(0,'timestamp',total_ts) #创建一个完整的dataframe,下面把df_data的值复制进去，再把没有的部分补上
        if len(df_data)<len(total_ts):
            x=len(df_data)-1
            start1=0
            while x>0 :
                  if df_date[x] != ts_date[x]:  
                      x-=1
                  elif df_date[x] == ts_date[x]: 
                       start1=x
                       break
            if start1>0:
               df_data1.iloc[0:start1,1:6]=df_data.iloc[0:start1,1:6]
               for j in range(start1,len(ts_date)):
                  if ts_date[j] in df_date:
                     loacte1=np.argwhere(df_date==ts_date[j])
                     df_data1.iloc[j,1:6]=df_data.iloc[int(loacte1[0]),1:6] 
                  else :
                       if j>0:
                          df_data1.iloc[j,1:6]=df_data1.iloc[j-1,1:6] 
            else:
                for j in range(0,len(ts_date)):
                   if ts_date[j] in df_date:
                      loacte1=np.argwhere(df_date==ts_date[j])
                      df_data1.iloc[j,1:6]=df_data.iloc[int(loacte1[0]),1:6] 
                   else :
                       if j>0:
                         df_data1.iloc[j,1:6]=df_data1.iloc[j-1,1:6] 
                      
        if len(df_data)>len(total_ts): 

           for j in range(0,len(ts_date)):
               if ts_date[j] in df_date:
                  loacte1=np.argwhere(df_date==ts_date[j])
                  df_data1.iloc[j,1:6]=df_data.iloc[int(loacte1[0]),1:6] 
               else :
                    if j>0:
                        df_data1.iloc[j,1:6]=df_data1.iloc[j-1,1:6] 
        df_data=df_data1         
    return df_data


if __name__ == '__main__':
    time_now = get_now_time()
    print(time_now)

    import time
    start = time.time()

    data = get_data('binance', 'btcusdt', '1d', '2017-10-01', '2019-12-05',0)
    print(data)

    print(time.time()-start)


'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
from uuid import uuid1

class Trade(object):
    __slots__ = [
        'context',
        'id',
        'order_id',
        'side',
        'price',
        'qty',
        'create_time'
    ]
    def __init__(self, context, order_id, side, price, qty):
        self.context = context
        self.id = str(uuid1())
        self.order_id = order_id
        self.side = side
        self.price = price
        self.qty= qty
        self.create_time = context.current_datetime()

    @property
    def amount(self):
        """
        :return: 成交金额
        """
        return self.price * self.qty

    @property
    def commission(self):
        """
        :return: 成交手续费
        """
        commission = 0
        if self.side == 'buy':
            commission = self.qty * self.context.commission['taker']
        elif self.side == 'sell':
            commission = self.price * self.qty * self.context.commission['taker']
        # print(commission)
        return commission

    def detail(self):
        return {
            'id':self.id,
            'order_id':self.order_id,
            'side':self.side,
            'price':self.price,
            'qty':self.qty,
            'amount':round(self.amount,6),
            'commission':round(self.commission,6),
            'create_time':self.create_time
        }

    def __getitem__(self, key, default=None):
        item_value = self.__getattribute__(key) if self.__getattribute__(key) else default
        return item_value

    def __repr__(self):
        content = ', '.join(['{}: {{}}'.format(item) for item in self.__slots__][1:]).format(
            self.id, self.order_id, self.price, self.qty, self.amount, self.commission,
            self.create_time)
        return 'Trade({})'.format(content)


'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
import os
from matplotlib.font_manager import FontProperties
# from vitu.data_api import API
from vitu.configuration import Config

# api = API(Config.tushare_token())

font = None
dirName = os.path.dirname(os.path.abspath(__file__))
font = FontProperties(fname=os.path.join(dirName, 'wqy-microhei.ttc'), size=10)


'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
#!/usr/bin/env python
#coding:utf-8

'''
取数据相关接口
'''
import datetime
import json
# import redis
import h5py
import sys

import pytz
import six
import numpy as np
import pandas as pd

from vitudata.apis import data_utils
from vitudata.config import Config
from vitudata.exceptions import ParamsError, SymbolNotSupported, InternalError
import os

ochlv_type = np.dtype(
    [('timestamp', 'uint64'), ('open', 'float_'), ('high', 'float_'), ('low', 'float_'), ('close', 'float_'),
     ('volume', 'float_')])

__all__ = [
    'get_bars',
]

def get_path(exchange, symbol, freq, year):
    dir = Config.h5_root_dir() + '/' + exchange.lower() + '/' + freq.lower() + '/' + symbol.replace('/', '')
    # dir = dir.lower()
    if not os.path.exists(dir):
        os.makedirs(dir, 0o755)
    return dir + '/' + str(year)

def __check_symbol_exists(symbol, exchange, freq):
    dir = Config.h5_root_dir() + '/' + exchange.lower()+ '/' + freq.lower() + '/' + symbol.replace('/', '')
    # dir = dir.lower()
    # print(dir)
    return os.path.exists(dir)


def __get_pos_of_h5(dt, frequency):
    if frequency == '1d':
        return dt.timetuple().tm_yday -1
    if frequency == '1m':
        return (dt.timetuple().tm_yday -1) * 1440 + dt.timetuple().tm_hour * 60  + dt.timetuple().tm_min
    if frequency == '5m':
        return (dt.timetuple().tm_yday - 1) * 288 + (dt.timetuple().tm_hour) * 12 + int(dt.timetuple().tm_min / 5)


def __get_redis_key(exchange, symbol, frequency):
    return f'ohlcv_{frequency}_{exchange}_{symbol.replace("/","").lower()}'


def get_bars(symbol, exchange, start_date=None, end_date=None, frequency='1d', timezone=1, count=None):
    '''
    获取标的行情数据

    Args:
        symbol: 标的
        exchange: 交易所
        start_date: 开始日期
        end_date: 结束日期
        frequency: 周期
        count: 取数据条数
    Returns:
        numpy.ndarray
    '''
    if count is not None and start_date is not None:
        raise ParamsError("get_bars 不能同时指定 start_date 和 count 两个参数")
    if not __check_symbol_exists(symbol, exchange, frequency):
        raise SymbolNotSupported(f"交易所{exchange}频率为{frequency}的交易对{symbol}不在支持范围内")
    now = datetime.datetime.now(pytz.timezone('utc'))
    differ=28800
    if timezone==1 or timezone is None:
        differ=28800 #默认北京与UTC的时差
    elif isinstance(timezone,str):
        now_utc=datetime.datetime.now(pytz.timezone('utc')).strftime('%Y-%m-%d %H:%M:%S')
        now_loacl=datetime.datetime.now(pytz.timezone(timezone)).strftime('%Y-%m-%d %H:%M:%S') #本地时间格式
        localstamp=datetime.datetime.strptime(now_loacl,'%Y-%m-%d %H:%M:%S').timestamp()
        utcstamp=datetime.datetime.strptime(now_utc,'%Y-%m-%d %H:%M:%S').timestamp()
        differ=round(localstamp-utcstamp)  #选取的timezone时间和utc时间的差（stamp按秒计算的差值）
    if end_date is None:
        end_dt = now
    else:
        end_dt = data_utils.convert_dt(end_date)

    # 根据频率确定准确时间
    if frequency == '1d':
        end_dt = end_dt.replace(hour=0, minute=0, second=0, microsecond=0)
    if frequency == '1m':
        end_dt = end_dt.replace(second=0, microsecond=0)
    if frequency == '5m':
        minu = end_dt.minute
        end_dt = end_dt.replace(minute=int(minu/5) * 5, second=0, microsecond=0)

    if start_date is None:
        if count is None:
            count = 100
        # 根据end_dt, count, frequency, 计算准确的start_dt
        if frequency == '1d':
            start_dt = end_dt - datetime.timedelta(days=count-1)
        if frequency == '1m':
            start_dt = end_dt - datetime.timedelta(minutes=count-1)
        if frequency == '5m':
            start_dt = end_dt - datetime.timedelta(minutes=(count-1)*5)
    else:
        start_dt = data_utils.convert_dt(start_date)
        if frequency == '1d':
            start_dt = start_dt.replace(hour=0, minute=0, second=0, microsecond=0)
        if frequency == '1m':
            start_dt = start_dt.replace(second=0, microsecond=0)
        if frequency == '5m':
            minu = start_dt.minute
            start_dt = start_dt.replace(minute=int(minu / 5) * 5, second=0, microsecond=0)

    start = start_dt
    result = None
    while True:
        end = start.replace(month=12, day=31)
        if frequency == '1m':
            end = end.replace(hour=23, minute=59)
        if frequency == '5m':
            end = end.replace(hour=23, minute=55)

        if end>end_dt:
            end = end_dt
        # read data bewteen start and end
        path = get_path(exchange, symbol, frequency, start.year)
        if os.path.exists(path):
            retry = 3
            i = __get_pos_of_h5(start, frequency)
            j = __get_pos_of_h5(end, frequency)
            while retry:
                try:
                    f = h5py.File(path, mode='r', libver='latest', swmr=True)
                    data = f['ohlcv'][i:j+1]
                    break
                except Exception as e:
                    retry -=1
                    if not retry:
                        raise InternalError("无法获取到数据，请稍后再试")
            if type(data) is np.ndarray and len(data) > 0:
                if result is None:
                    result = data
                else:
                    result = np.concatenate((result,data),axis=0)
        if end >= end_dt:
            break
        start = end.replace(hour=0, minute=0) + datetime.timedelta(days=1)
    result = result[result['timestamp'] >= 0] #保留timestamp 和ohlcv为0的那些行

    result['timestamp']=result['timestamp']-differ
    #换回成本地时间的时间戳
   

    # 判断是否有数据在redis中而不在h5中
    # if len(result) > 0:
    #     min_t = result[-1][0]
    # else:
    #     min_t = start_dt.timestamp() -1
    # max_t = end_dt.timestamp()
    # if (frequency == '1m' or frequency == '5m') and (now - end_dt).total_seconds() < 7200:
    #     #最近2小时的数据在redis中
    #     if frequency == '5m':
    #         max = 24
    #     else:
    #         max = 120
    #     rkey = __get_redis_key(exchange, symbol, frequency)
    #     r = redis.Redis(host=Config.redis_host(), port=Config.redis_port())
    #     data = r.lrange(rkey, 0, max)
    #     jd = [json.loads(i) for i in data]
    #     jd = [(int(i['t']), float(i['o']), float(i['h']), float(i['l']), float(i['c']), float(i['v'])) for i in jd]
    #     npdata = np.asarray(jd, dtype=ochlv_type)
    #     npdata = npdata[npdata['timestamp'] > min_t]
    #     npdata = npdata[npdata['timestamp'] <= max_t]
    #     result = np.concatenate((result, npdata), axis=0)
    return result

if __name__ == '__main__':
    print(get_bars('btcusdt', 'binance', frequency='1d', start_date='2019-12-30', end_date='2020-1-3'))






'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
# import pandas as pd
# import numpy as np
#
# pd.set_option('display.max_rows', None)  # 让行显示完
# pd.set_option('display.max_columns', None)  # 让列显示完
# # pd.set_option('display.height',1000)               # 高度最大显示1000行
# pd.set_option('display.width', 1000)  # 宽度最大显示1000列
# exchange = 'binance'
# df_data = pd.read_csv('./{}_min_order.csv'.format(exchange))
# print(df_data)
# a = df_data[['baseAsset', 'quoteAsset', 'minTradeAmount', 'minOrderValue']]
# b = np.array(a)
# print(b)
# for i in b:
#     min_order['binance'][i[0] + '/' + i[1]] = {'min_order_qty': i[2],
#                                                'min_order_amount': i[3]}
# print(min_order)

min_order = {'binance': {'ETH/BTC': {'min_order_qty': 0.001, 'min_order_amount': 0.0001},
                         'LTC/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'BNB/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'NEO/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'QTUM/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'EOS/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'SNT/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'BNT/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'GAS/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'BNB/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'BTC/USDT': {'min_order_qty': 1e-06, 'min_order_amount': 10.0},
                         'ETH/USDT': {'min_order_qty': 1e-05, 'min_order_amount': 10.0},
                         'OAX/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'DNT/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'MCO/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'MCO/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'WTC/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'WTC/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'LRC/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'LRC/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'QTUM/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'YOYO/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'OMG/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'OMG/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'ZRX/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'ZRX/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'STRAT/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'STRAT/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'SNGLS/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'SNGLS/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'BQX/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'BQX/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'KNC/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'KNC/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'FUN/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'FUN/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'SNM/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'SNM/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'NEO/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'IOTA/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'IOTA/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'LINK/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'LINK/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'XVG/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'XVG/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'MDA/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'MDA/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'MTL/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'MTL/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'EOS/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'SNT/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'ETC/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'ETC/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'MTH/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'MTH/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'ENG/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'ENG/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'DNT/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'ZEC/BTC': {'min_order_qty': 0.001, 'min_order_amount': 0.0001},
                         'ZEC/ETH': {'min_order_qty': 0.001, 'min_order_amount': 0.01},
                         'BNT/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'AST/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'AST/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'DASH/BTC': {'min_order_qty': 0.001, 'min_order_amount': 0.0001},
                         'DASH/ETH': {'min_order_qty': 0.001, 'min_order_amount': 0.01},
                         'OAX/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'BTG/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'BTG/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'EVX/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'EVX/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'REQ/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'REQ/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'VIB/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'VIB/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'TRX/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'TRX/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'POWR/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'POWR/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'ARK/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'ARK/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'YOYO/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'XRP/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'XRP/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'ENJ/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'ENJ/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'STORJ/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'STORJ/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'BNB/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'YOYO/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'POWR/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'KMD/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'KMD/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'NULS/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'RCN/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'RCN/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'RCN/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'NULS/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'NULS/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'RDN/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'RDN/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'RDN/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'XMR/BTC': {'min_order_qty': 0.001, 'min_order_amount': 0.0001},
                         'XMR/ETH': {'min_order_qty': 0.001, 'min_order_amount': 0.01},
                         'DLT/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'WTC/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'DLT/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'DLT/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'AMB/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'AMB/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'AMB/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'BAT/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'BAT/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'BAT/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'BCPT/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'BCPT/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'BCPT/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'ARN/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'ARN/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'GVT/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'GVT/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'CDT/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'CDT/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'GXS/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'GXS/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'NEO/USDT': {'min_order_qty': 0.001, 'min_order_amount': 10.0},
                         'NEO/BNB': {'min_order_qty': 0.01, 'min_order_amount': 0.1},
                         'POE/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'POE/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'QSP/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'QSP/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'QSP/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'BTS/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'BTS/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'BTS/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'XZC/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'XZC/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'XZC/BNB': {'min_order_qty': 0.01, 'min_order_amount': 0.1},
                         'LSK/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'LSK/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'LSK/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'TNT/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'TNT/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'FUEL/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'MANA/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'MANA/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'BCD/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'BCD/ETH': {'min_order_qty': 0.001, 'min_order_amount': 0.01},
                         'DGD/BTC': {'min_order_qty': 0.001, 'min_order_amount': 0.0001},
                         'DGD/ETH': {'min_order_qty': 0.001, 'min_order_amount': 0.01},
                         'IOTA/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'ADX/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'ADX/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'ADX/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'ADA/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'ADA/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'PPT/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'PPT/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'CMT/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'CMT/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'CMT/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'XLM/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'XLM/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'XLM/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'CND/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'CND/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'CND/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'LEND/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'LEND/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'WABI/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'WABI/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'WABI/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'LTC/ETH': {'min_order_qty': 0.001, 'min_order_amount': 0.01},
                         'LTC/USDT': {'min_order_qty': 1e-05, 'min_order_amount': 10.0},
                         'LTC/BNB': {'min_order_qty': 0.001, 'min_order_amount': 0.1},
                         'TNB/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'TNB/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'WAVES/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'WAVES/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'WAVES/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'GTO/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'GTO/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'GTO/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'ICX/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'ICX/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'ICX/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'OST/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'OST/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'OST/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'ELF/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'ELF/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'AION/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'AION/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'AION/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'NEBL/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'NEBL/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'NEBL/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'BRD/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'BRD/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'BRD/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'MCO/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'EDO/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'EDO/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'NAV/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'NAV/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'NAV/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'LUN/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'APPC/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'APPC/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'APPC/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'VIBE/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'VIBE/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'RLC/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'RLC/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'RLC/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'INS/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'INS/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'PIVX/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'PIVX/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'PIVX/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'IOST/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'IOST/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'STEEM/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'STEEM/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'STEEM/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'NANO/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'NANO/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'NANO/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'VIA/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'VIA/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'VIA/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'BLZ/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'BLZ/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'BLZ/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'AE/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'AE/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'AE/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'NCASH/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'NCASH/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'POA/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'POA/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'POA/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'ZIL/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'ZIL/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'ZIL/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'ONT/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'ONT/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'ONT/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'STORM/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'STORM/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'STORM/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'QTUM/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'QTUM/USDT': {'min_order_qty': 0.001, 'min_order_amount': 10.0},
                         'XEM/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'XEM/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'XEM/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'WAN/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'WAN/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'WAN/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'WPR/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'WPR/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'QLC/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'QLC/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'SYS/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'SYS/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'SYS/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'QLC/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'GRS/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'GRS/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'ADA/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'ADA/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'GNT/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'GNT/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'GNT/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'LOOM/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'LOOM/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'LOOM/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'XRP/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'REP/BTC': {'min_order_qty': 0.001, 'min_order_amount': 0.0001},
                         'REP/ETH': {'min_order_qty': 0.001, 'min_order_amount': 0.01},
                         'REP/BNB': {'min_order_qty': 0.01, 'min_order_amount': 0.1},
                         'BTC/TUSD': {'min_order_qty': 1e-06, 'min_order_amount': 10.0},
                         'ETH/TUSD': {'min_order_qty': 1e-05, 'min_order_amount': 10.0},
                         'ZEN/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'ZEN/ETH': {'min_order_qty': 0.001, 'min_order_amount': 0.01},
                         'ZEN/BNB': {'min_order_qty': 0.01, 'min_order_amount': 0.1},
                         'SKY/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'SKY/ETH': {'min_order_qty': 0.001, 'min_order_amount': 0.01},
                         'SKY/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'EOS/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'EOS/BNB': {'min_order_qty': 0.01, 'min_order_amount': 0.1},
                         'CVC/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'CVC/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'CVC/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'THETA/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'THETA/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'THETA/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'XRP/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'TUSD/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'IOTA/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'XLM/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'IOTX/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'IOTX/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'QKC/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'QKC/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'AGI/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'AGI/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'AGI/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'NXS/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'NXS/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'NXS/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'ENJ/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'DATA/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'DATA/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'ONT/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'TRX/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'TRX/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'ETC/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'ETC/BNB': {'min_order_qty': 0.01, 'min_order_amount': 0.1},
                         'ICX/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'SC/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'SC/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'SC/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'NPXS/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'KEY/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'KEY/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'NAS/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'NAS/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'NAS/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'MFT/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'MFT/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'MFT/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'DENT/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'ARDR/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'ARDR/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'ARDR/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'NULS/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'HOT/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'HOT/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'VET/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'VET/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'VET/USDT': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'VET/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'DOCK/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'DOCK/ETH': {'min_order_qty': 1.0, 'min_order_amount': 0.01},
                         'POLY/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'POLY/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'HC/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'HC/ETH': {'min_order_qty': 0.01, 'min_order_amount': 0.01},
                         'GO/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'GO/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'PAX/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'RVN/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'RVN/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'DCR/BTC': {'min_order_qty': 0.001, 'min_order_amount': 0.0001},
                         'DCR/BNB': {'min_order_qty': 0.001, 'min_order_amount': 0.1},
                         'MITH/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'MITH/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'BCH/BTC': {'min_order_qty': 0.001, 'min_order_amount': 0.0001},
                         'BCH/USDT': {'min_order_qty': 1e-05, 'min_order_amount': 10.0},
                         'BNB/PAX': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'BTC/PAX': {'min_order_qty': 1e-06, 'min_order_amount': 10.0},
                         'ETH/PAX': {'min_order_qty': 1e-05, 'min_order_amount': 10.0},
                         'XRP/PAX': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'EOS/PAX': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'XLM/PAX': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'REN/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'REN/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'BNB/TUSD': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'XRP/TUSD': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'EOS/TUSD': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'XLM/TUSD': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'BNB/USDC': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'BTC/USDC': {'min_order_qty': 1e-06, 'min_order_amount': 10.0},
                         'ETH/USDC': {'min_order_qty': 1e-05, 'min_order_amount': 10.0},
                         'XRP/USDC': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'EOS/USDC': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'XLM/USDC': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'USDC/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'ADA/TUSD': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'TRX/TUSD': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'NEO/TUSD': {'min_order_qty': 0.001, 'min_order_amount': 10.0},
                         'TRX/XRP': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'XZC/XRP': {'min_order_qty': 0.001, 'min_order_amount': 10.0},
                         'PAX/TUSD': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'USDC/TUSD': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'USDC/PAX': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'LINK/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'LINK/TUSD': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'LINK/PAX': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'LINK/USDC': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'WAVES/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'WAVES/TUSD': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'WAVES/USDC': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'BCH/TUSD': {'min_order_qty': 1e-05, 'min_order_amount': 10.0},
                         'BCH/PAX': {'min_order_qty': 1e-05, 'min_order_amount': 10.0},
                         'BCH/USDC': {'min_order_qty': 1e-05, 'min_order_amount': 10.0},
                         'LTC/TUSD': {'min_order_qty': 1e-05, 'min_order_amount': 10.0},
                         'LTC/PAX': {'min_order_qty': 1e-05, 'min_order_amount': 10.0},
                         'LTC/USDC': {'min_order_qty': 1e-05, 'min_order_amount': 10.0},
                         'TRX/PAX': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'TRX/USDC': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'BTT/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'BTT/USDT': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'BNB/USDS': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'BTC/USDS': {'min_order_qty': 1e-06, 'min_order_amount': 10.0},
                         'USDS/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'USDS/PAX': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'USDS/TUSD': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'USDS/USDC': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'BTT/PAX': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'BTT/TUSD': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'BTT/USDC': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'ONG/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'ONG/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'ONG/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'HOT/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'HOT/USDT': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'ZIL/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'ZRX/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'ZRX/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'FET/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'FET/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'FET/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'BAT/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'XMR/BNB': {'min_order_qty': 0.001, 'min_order_amount': 0.1},
                         'XMR/USDT': {'min_order_qty': 1e-05, 'min_order_amount': 10.0},
                         'ZEC/BNB': {'min_order_qty': 0.001, 'min_order_amount': 0.1},
                         'ZEC/USDT': {'min_order_qty': 1e-05, 'min_order_amount': 10.0},
                         'ZEC/PAX': {'min_order_qty': 1e-05, 'min_order_amount': 10.0},
                         'ZEC/TUSD': {'min_order_qty': 1e-05, 'min_order_amount': 10.0},
                         'ZEC/USDC': {'min_order_qty': 1e-05, 'min_order_amount': 10.0},
                         'IOST/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'IOST/USDT': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'CELR/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'CELR/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'CELR/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'ADA/PAX': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'ADA/USDC': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'NEO/PAX': {'min_order_qty': 0.001, 'min_order_amount': 10.0},
                         'NEO/USDC': {'min_order_qty': 0.001, 'min_order_amount': 10.0},
                         'DASH/BNB': {'min_order_qty': 0.001, 'min_order_amount': 0.1},
                         'DASH/USDT': {'min_order_qty': 1e-05, 'min_order_amount': 10.0},
                         'NANO/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'OMG/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'OMG/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'THETA/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'ENJ/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'MITH/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'MATIC/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'MATIC/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'MATIC/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'ATOM/BNB': {'min_order_qty': 0.01, 'min_order_amount': 0.1},
                         'ATOM/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'ATOM/USDT': {'min_order_qty': 0.001, 'min_order_amount': 10.0},
                         'ATOM/USDC': {'min_order_qty': 0.001, 'min_order_amount': 10.0},
                         'ATOM/PAX': {'min_order_qty': 0.001, 'min_order_amount': 10.0},
                         'ATOM/TUSD': {'min_order_qty': 0.001, 'min_order_amount': 10.0},
                         'ETC/USDC': {'min_order_qty': 0.001, 'min_order_amount': 10.0},
                         'ETC/PAX': {'min_order_qty': 0.001, 'min_order_amount': 10.0},
                         'ETC/TUSD': {'min_order_qty': 0.001, 'min_order_amount': 10.0},
                         'BAT/USDC': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'BAT/PAX': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'BAT/TUSD': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'PHB/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'PHB/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'PHB/TUSD': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'TFUEL/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'TFUEL/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'TFUEL/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'ONE/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'ONE/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'ONE/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'ONE/USDC': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'FTM/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'FTM/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'FTM/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'FTM/USDC': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'ALGO/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'ALGO/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'ALGO/USDT': {'min_order_qty': 0.001, 'min_order_amount': 10.0},
                         'ALGO/TUSD': {'min_order_qty': 0.001, 'min_order_amount': 10.0},
                         'ALGO/PAX': {'min_order_qty': 0.001, 'min_order_amount': 10.0},
                         'ALGO/USDC': {'min_order_qty': 0.001, 'min_order_amount': 10.0},
                         'GTO/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'ERD/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'ERD/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'ERD/USDT': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'DOGE/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'DOGE/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'DOGE/USDT': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'DUSK/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'DUSK/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'DUSK/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'DUSK/USDC': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'DUSK/PAX': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'BGBP/USDC': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'ANKR/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'ANKR/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'ANKR/USDT': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'ONT/PAX': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'ONT/USDC': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'WIN/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'WIN/USDT': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'WIN/USDC': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'COS/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'COS/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'COS/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'NPXS/USDT': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'NPXS/USDC': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'COCOS/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'COCOS/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'COCOS/USDT': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'MTL/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'TOMO/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'TOMO/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'TOMO/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'TOMO/USDC': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'PERL/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'PERL/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'PERL/USDC': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'PERL/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'DENT/USDT': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'MFT/USDT': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'KEY/USDT': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'STORM/USDT': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'DOCK/USDT': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'WAN/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'FUN/USDT': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'CVC/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'BTT/TRX': {'min_order_qty': 0.1, 'min_order_amount': 0.001},
                         'WIN/TRX': {'min_order_qty': 0.1, 'min_order_amount': 0.001},
                         'CHZ/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'CHZ/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'CHZ/USDT': {'min_order_qty': 1.0, 'min_order_amount': 10.0},
                         'BAND/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'BAND/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'BAND/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'BNB/BUSD': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'BTC/BUSD': {'min_order_qty': 1e-06, 'min_order_amount': 10.0},
                         'BUSD/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'BEAM/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'BEAM/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'BEAM/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'XTZ/BNB': {'min_order_qty': 0.1, 'min_order_amount': 0.1},
                         'XTZ/BTC': {'min_order_qty': 0.01, 'min_order_amount': 0.0001},
                         'XTZ/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'REN/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'RVN/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'HC/USDT': {'min_order_qty': 0.01, 'min_order_amount': 10.0},
                         'HBAR/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'HBAR/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'HBAR/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0},
                         'NKN/BNB': {'min_order_qty': 1.0, 'min_order_amount': 0.1},
                         'NKN/BTC': {'min_order_qty': 1.0, 'min_order_amount': 0.0001},
                         'NKN/USDT': {'min_order_qty': 0.1, 'min_order_amount': 10.0}}}




'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
from vitu.report.report import Report

class SimpleReport(Report):
    def __init__(self, portfolio):
        super(SimpleReport, self).__init__(portfolio)
        self.cumulative_returns = None
        self.benchmark_cumulative_returns = None
        self.returns_rate = None
        self.benchmark_returns_rate = None

        self.excess_return = None
        self.log_returns = None
        self.benchmark_log_returns = None

    def get_dates_values(self):
        """
        获取dates和values
        :return: dates和values
        """
        values = list()
        dates = list()
        benchmarks=self.benchmark 
        values.append(self.context.init_position_total)
        for date, pf in self.rebalance_history.items():
            values.append(sum([asset['total_amount'] for asset in pf['portfolio_position']['detail'].values()]))
            dates.append(date)
    
        date1=[]
        value1=[]
        for i, v in enumerate(dates):
            if v in benchmarks['timestamp'].values:
               date1.append(dates[i])
               value1.append(values[i])
        dates=date1
        values=value1
        return dates,values

    def run(self):
        self.dates,self.values = self.get_dates_values()
        self.bm_values = self.get_benchmark_values(self.dates)
        # print(self.values)

        st_returns = self.get_relative_returns(self.values)
        bm_returns = self.get_relative_returns(self.bm_values)
        # print(st_returns)

        self.cumulative_returns, self.returns_rate = self.get_cumulative_returns(st_returns)
        self.benchmark_cumulative_returns, self.benchmark_returns_rate = self.get_cumulative_returns(bm_returns)

        index = len(self.benchmark_returns_rate)
        self.excess_return = [round((self.returns_rate[-index:][i]-self.benchmark_returns_rate[i]),4)
                              for i in range(len(self.benchmark_returns_rate))]

        self.log_returns = self.get_log_returns(self.values)
        self.benchmark_log_returns = self.get_log_returns(self.bm_values)

        # 计算高级指标
        self.annualized_return = self.get_annualized_return(self.cumulative_returns)
        self.benchmark_annualized_return = self.get_annualized_return(self.benchmark_cumulative_returns)
        self.max_drawdown = self.get_max_drawdown(self.cumulative_returns)
        self.annualized_volatility = self.get_annualized_volatility(st_returns)
        rf = self.get_riskfree_rate()
        self.alpha, self.beta = self.get_CAPM(st_returns, bm_returns, rf)
        self.sharpe = self.get_sharpe(st_returns, rf)
        self.information_ratio = self.get_information_ratio(st_returns, bm_returns)
        netvalues = self.cumulative_returns
        max1 = []
        for i, v in enumerate(netvalues):
            j = max(netvalues[:i + 1])
            if j == v:
                max1.append(0)
            else:
                max1.append(1 - v / j)
        if max(max1)>0:
           a1 = max1.index(max(max1))
           a2 = netvalues.index(max(netvalues[:a1]))
           mdate1 = self.dates[a2]
           mdate2 = self.dates[a1]
           maxdown_interval = [mdate1, mdate2]
        else:
            maxdown_interval = [self.dates[0], self.dates[0]] 

        report = {
            "display_type": "strategy_overview",
            "Date": self.dates,
            "cumulative_returns": self.returns_rate,
            "netvalues": self.cumulative_returns,
            "Netvalue": self.cumulative_returns[-1],
            "benchmark_cumulative_returns": self.benchmark_returns_rate,
            "excess_return": self.excess_return,
            "log_returns": self.log_returns,
            "benchmark_log_returns": self.benchmark_log_returns,
            # 风险指标
            "annualized_return": self.annualized_return,  # 策略 年化收益
            "benchmark_annualized_return": self.benchmark_annualized_return,  # 基准 年化收益
            "cumulative_return": self.returns_rate[-1],  # 策略 累计收益
            "benchmark_cumulative_return": self.benchmark_returns_rate[-1],  # 基准 累计收益
            "alpha": self.alpha,
            "sharpe": self.sharpe,
            "information_ratio": self.information_ratio,
            "winning_ratio": '' if not self.winning_ratio else '',
            "beta": self.beta,
            "annualized_volatility": self.annualized_volatility,
            "max_drawdown": self.max_drawdown,
            "maxdown_interval": maxdown_interval
        }
        return report




'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
from uuid import uuid1

class Order(object):
    __slots__ = [
        'context',
        'id',
        'account',
        'instrument',
        'exchange',
        'asset_class',
        'side',
        'type',
        'limit_price',
        'avg_price',
        'initial_qty',
        'filled_qty',
        'filled_amount',
        '_commission',
        'status',
        'create_time',
        'complete_time',
        'trades'
    ]
    def __init__(self, context, name, symbol, exchange, asset_class, side, type, price, qty):
        """
        :param name: 'spot_account'
        :param symbol: 'btc/usdt'
        :param exchange: 'binance'
        :param asset_class: 'spot' | 'contract'
        :param side: 'sell' | 'buy' | 'open_long' | 'close_long'...
        :param type: 'limit' | 'market'
        :param price: 价格
        :param qty: 数量
        """
        self.context = context
        self.id = str(uuid1())
        self.account = name
        self.instrument = symbol.lower()
        self.exchange = exchange
        self.asset_class = asset_class
        self.side = side
        self.type = type
        self.limit_price = price       # 限价单
        self.avg_price = None
        self.initial_qty = qty      # 下单数量, 以btc/usdt为例, 如果是限价,买卖均是btc的数量, 如果是市价,卖是btc数量,买是usdt数量
        self.filled_qty = None         # 成交数量, 以btc/usdt为例, 无论什么方向,都是btc的数量
        self.filled_amount = None      # 成交金额, 无论什么方向,都是usdt的数量
        self._commission = 0
        self.status = 'Pending'        # 'Filled' | 'Partial_Filled' | 'Pending' | 'Cancelled' | 'Withdrawn'
        self.create_time = context.current_datetime()
        self.complete_time = None
        self.trades = list()

    @property
    def commission(self):
        """
        :return: 成交手续费
        """
        return self._commission

    @commission.setter
    def commission(self, value):
        """
        :return: 成交手续费
        """
        if not value:
            value = 0
        self._commission += value

    def trade_update(self, trade):
        if not self.avg_price:
            self.avg_price = 0
        if not self.filled_qty:
            self.filled_qty = 0
        self.avg_price = trade['price']
        self.filled_qty = trade['qty']
        self.filled_amount = self.avg_price * self.filled_qty
        self.commission = trade['commission']
        self.status = 'Filled'
        self.complete_time = self.context.current_datetime()
        self.trades.append(trade.detail())
        # self.trades.append(trade)

    def detail(self):
        return {
            'id':self.id,
            'account':self.account,
            'instrument':self.instrument.upper(),
            'exchange':self.exchange,
            'asset_class':self.asset_class,
            'side':self.side,
            'type':self.type,
            'limit_price': self.limit_price,
            'avg_price': self.avg_price,
            'initial_qty': self.initial_qty,
            'filled_qty': self.filled_qty,
            'qty_unit': self.instrument.split('/')[0],
            'filled_amount': round(self.filled_amount,6),  # 成交金额
            'amount_unit': self.instrument.split('/')[1],
            'commission': round(self.commission,6),  # 手续费
            'commission_unit':self.instrument.split('/')[0] if self.side == 'buy' else self.instrument.split('/')[1],
            'status': self.status,
            'create_time': self.create_time,
            'complete_time': self.complete_time,
            'trades': self.trades
        }

    def __getitem__(self, key, default=None):
        item_value = self.__getattribute__(key) if self.__getattribute__(key) else default
        return item_value

    def __repr__(self):
        content = ', '.join(['{}: {{}}'.format(item) for item in self.__slots__[1:]]).format(
            self.id, self.account, self.instrument, self.exchange, self.asset_class, self.side,
            self.type, self.limit_price, self.avg_price, self.initial_qty, self.filled_qty,
            self.filled_amount, self.commission, self.status, self.create_time,
            self.complete_time, self.trades
        )
        return 'Order({})'.format(content)

if __name__ == '__main__':
    order1 = Order('111212', 'btc/usdt', 'binance', 'spot', 'sell', 'limit', 1200,2)
    print(order1.detail())


import traceback
import pandas as pd
import datetime
import time
import matplotlib.pyplot as plt
import xlwt
import os 

from vitu.account.account_manager import AccountManager
from vitu.trade.portfoilo.portfolio import Portfolio
from vitu.trade.position.portfolio_position import PortfolioPosition
from vitu.report.simple_report import SimpleReport
from vitu.report.complete_report import CompleteReport
from vitu.utils.date_utils import str2timestamp, get_total_dates, str2datetime,get_day_dates
from vitu.utils.log_utils import logger
from vitu.utils.min_qty import get_min_order
from vitu.utils.output_utils import output
import numpy as np
 
np.set_printoptions(threshold=100000)
pd.set_option('display.max_columns',100000)
pd.set_option('display.width', 100000)
pd.set_option('display.max_colwidth',100000)


class Strategy(object):
    def __new__(cls, *args, **kwargs):
        if not hasattr(cls, '_instance'):
            _instance = super(Strategy, cls).__new__(cls)
            cls._instance = _instance
        return cls._instance

    def __init__(self, initialize=None, handle_data=None, universe=None,
                 benchmark=None, freq=None, refresh_rate=None):
        """
        :param initialize: initialize
        :param handle_data: handle_data
        :param universe: ('BTC/USDT.okex', 'ETH/BTC.okex', 'XBT/USD.bitmex')
        :param benchmark: 'csi5'          # 策略参考标准
        :param freq: 'd',                 # 'd'日线回测，'m'15分钟线回测
        :param refresh_rate: 1 or (1,['08:00:00']) or (1,['08:00:00','18:00:00']),
        # 调仓时间间隔，若freq = 'd'的单位为交易日，若freq = 'm'时间间隔为分钟
        :param trigger_time: '08:00'      # 日级别调仓时间,时区为UTC
        """
        self.initialize = initialize
        self.handle_data = handle_data
        self.universe = universe
        self.benchmark = benchmark
        self.frequency = freq
        # self.refresh_rate = refresh_rate
        self.refresh_rate = refresh_rate[0] if isinstance(refresh_rate, (tuple)) else refresh_rate
        self.trigger_time = refresh_rate[1] if isinstance(refresh_rate, (tuple)) else None

        self.start = None
        self.end = None
        self.commission = None
        # self.asset_varieties = None
        self.portfolio = None
        self.context = None
        self.cache_data = None

        self.universe_assets = None
        self.all_assets = None

        self.total_dates = None
        self.strategy_dates = None

    def _initialize(self, strategy=None, start=None, end=None, timezone=None, commission=None):
        self.start = start
        self.end = end
        self.commission = commission
        self.timezone=timezone
        self.portfolio = Portfolio(strategy, start, end, commission,
                                   self.frequency, self.refresh_rate, self.trigger_time)
        self.context = self.portfolio.context

        # 最小精度
        self.context.min_order = {}
        for univ in self.universe:
            exchange = univ.split('.')[1]
            self.context.min_order[exchange] = {}
        # print(self.context.min_order)
        for univ in self.universe:
            exchange = univ.split('.')[1]
            symbol = univ.split('.')[0].lower()
            min_order_qty, min_order_amount = get_min_order(exchange, symbol)
            self.context.min_order[exchange][symbol] = {'min_order_qty':min_order_qty,
                                                             'min_order_amount':min_order_amount}


        # 记录universe所有的asset
        self.universe_assets = list()
        for univ in self.universe:
            base_asset = univ.split('.')[0].split('/')[0].lower()
            quote_asset = univ.split('.')[0].split('/')[1].lower()
            if base_asset not in self.universe_assets:
                self.universe_assets.append(base_asset)
            if quote_asset not in self.universe_assets:
                self.universe_assets.append(quote_asset)
        self.all_assets = list(set(AccountManager().asset_varieties + self.universe_assets))
        # print(self.all_assets)

        # # 初始每个账户添加除初始化之外的universe的asset
        # for name,account in self.portfolio.accounts.items():
        #     for asset in universe_assets:
        #         if asset not in account.current_position.keys():
        #             account.current_position[asset] = SpotPosition('spot', asset, 0, 0, 0, 0)

        self.total_dates = get_total_dates(self.frequency, 1, self.trigger_time, self.start, self.end)
        self.strategy_dates = get_total_dates(self.frequency, self.refresh_rate, self.trigger_time, self.start, self.end)
        if self.frequency in ['d','1d','day','1day']:
           self.day_date=get_day_dates(self.start, self.end,self.refresh_rate)
        else:
           self.day_date=get_day_dates(self.start, self.end)
        # 先输出report的dates信息，画图用
        #output({"display_type": "strategy",
        #     "dates":self.day_date})

        try:
            self.initialize(self.context)
        except Exception:
            output({"display_type": "error",
                    "error_msg": traceback.format_exc()})
            return 1

    async def _handle_data(self):
        # print("******************************************************************")
        clock = self.context.clock
        current_date = clock.current_date
        logger.current_date = current_date
        current_timestamp = clock.current_timestamp
        # print(current_date)
        prebars=clock.pre_bar

        pre_start = str(str2datetime(self.start) - datetime.timedelta(days=prebars))
        try:
            if not self.cache_data:
                self.cache_data = self.context.prepare_data(self.universe, self.all_assets, self.benchmark,
                                                                  self.frequency, pre_start, self.end, self.timezone)
        except Exception:
            output({"display_type": "error",
                    "error_msg": traceback.format_exc()})
            return 1

        # 记录初始持仓
        start = self.start + ' ' + self.trigger_time[0] if self.trigger_time else self.start # 只支持一个trigger_time
        if current_timestamp == str2timestamp(start):
            for name, account in self.portfolio.accounts.items():
                for asset, position in account.current_position.items():
                    position.context = self.context
            init_portfolio_position = dict()
            for asset in self.context.asset_varieties:
                init_pp_detail = PortfolioPosition(self.context, asset, self.portfolio.accounts).detail()
                init_portfolio_position[asset] = init_pp_detail

            self.context.init_portfolio_position = init_portfolio_position
            self.context.init_position_total = sum([asset['total_amount'] for asset in init_portfolio_position.values()])

            self.context.init_total_account_position = {}
            for name in self.context.accounts_name:
                name_position = "{}_position".format(name)
                total_amount = 0
                for asset,value in self.context.init_portfolio_position.items():
                    if name in value['consist_of'].keys():
                        total_amount += value['consist_of'][name]['amount']
                self.context.init_total_account_position[name_position] = total_amount

            # logger.info('【记录初始持仓时间】：{} s'.format(str(time.time() - start2)[:5]))

        try:
            if str(current_date) in self.strategy_dates:
                self.handle_data(self.context)
        except Exception:
            output({"display_type": "error",
                    "error_msg": traceback.format_exc()})
            return 1

        self.portfolio.record_history()

        # logger.info('【_handle_data耗时】：{} s'.format(str(time.time() - start2)[:5]))
        # print("******************************************************************", end='\n\n')

    def simple_report(self):
        report = SimpleReport(self.portfolio)
        simple_report = report.run()
        output(simple_report)

    def complete_report(self):
        report = CompleteReport(self.portfolio)
        complete_report = report.run()
        output(complete_report)

    def simplereport_excel(self):
        report = SimpleReport(self.portfolio)
        simple_report = report.run()
        output(simple_report)
        # 回测报告以excel表格形式输出
        workbook=xlwt.Workbook(encoding='utf-8')
        sheet1=workbook.add_sheet('sheet1',cell_overwrite_ok=True)
        c=0
        for key,value in simple_report.items():
            sheet1.write(0,c,key)
            if isinstance(value,list):
               s1=len(value)
               for s2 in range(0,s1):
                   temp=value[s2]
                   sheet1.write(s2+1,c,temp)  
            else:
                 sheet1.write(1,c,value)         
            c+=1
        root1=os.getcwd()    #获得当前目录路径，用来存放回测报告
        adress1=root1+'/'+'simple_report.xls' #保存报告的地址
        print(adress1)
        workbook.save(adress1)     
        #净值图形输出
        plt.plot(simple_report["cumulative_returns"])
        plt.title("Cumulative_returns",fontsize='large')
        plt.show()
        
    

if __name__ == '__main__':
    import time
    a = time.time()

    df_data = pd.read_csv('../data/static/binance_min_order.csv')
    print(df_data)

    symbol = 'BTC/USDT'
    base_asset = symbol.split('/')[0]
    quote_asset = symbol.split('/')[1]

    # 最小下单数量 0.000001BTC
    min_order_qty = df_data.loc[(df_data['baseAsset'] == base_asset) & (df_data['quoteAsset'] == quote_asset)][
        'minTradeAmount'].tolist()[0]
    print(min_order_qty)
    print(type(min_order_qty))
    # 最小下单金额 10USDT
    min_order_amount = df_data.loc[(df_data['baseAsset'] == base_asset) & (df_data['quoteAsset'] == quote_asset)][
        'minOrderValue'].tolist()[0]
    print(min_order_amount)
    print(type(min_order_amount))

    print(time.time()-a)



'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
import datetime
import numpy as np

from vitu.data.cache import DataCache
from vitu.context.clock import Clock
from vitu.utils.data_utils import get_ohlcv_data
from vitu.utils.date_utils import (
    timestamp2str,
    str2timestamp,
    datetime2timestamp
)

class Context(object):
    def __new__(cls, *args, **kwargs):
        if not hasattr(cls, '_instance'):
            _instance = super(Context, cls).__new__(cls)
            cls._instance = _instance
        return cls._instance

    def __init__(self, portfolio, start_date=None, end_date=None, commission=None,
                 frequency=None, refresh_rate=None, trigger_time=None):
        """
        context包含运行时间、回测参数、回测运行所需数据 佣金等等
        """
        self.clock = Clock(start_date, end_date, frequency, refresh_rate, trigger_time)
        self.cacher = DataCache()
        self.portfolio = portfolio
        self.commission = commission
        self.commission 
        self.frequency = frequency

        self.init_portfolio_position = None
        self.init_position_total = None
        self.init_total_account_position = None

        self.completed_time = None
        self.accounts_name = None    # 记录account_name
        self.asset_varieties = None  # 记录asset种类


    def get_account(self, name):
        return self.portfolio.get_account(name)

    def current_datetime(self):
        return str(self.clock.current_date)

    def previous_datetime(self):
        return str(self.clock.previous_date)

    def prepare_data(self, universe=None, all_assets=None, benchmark=None, frequency=None,
                            pre_start_date=None, end_date=None,timezone=None):
        ohlcv_data = get_ohlcv_data(universe, all_assets, benchmark, frequency, pre_start_date, end_date,timezone)
        self.cacher.data = ohlcv_data
        # print(ohlcv_data)
        return self.cacher.data

    def history(self, symbol=None, attributes=None, bars=None, rtype='ndarray'):
        """
        :param symbol: 'BTC/USDT.binance'
        :param attributes: 'close'
        :param bars: 30
        :param rtype: 'ndarray'/'dataframe'/'list'
        :return: data
        ：frequency
        """
        k_lines_key = symbol.split('.')[1] + '-spot-' + symbol.split('.')[0].replace('/', '').lower()
        freq1=self.frequency
        if freq1 in ['d','1d','day','1day','daily']:
            end = self.clock.bars_start + self.clock.pre_bar
            start =end-bars              
        elif freq1 in ['m','1m','min','1minute']:
             end = self.clock.bars_start + self.clock.pre_bar*1440
             start =end-bars
        elif freq1 in ['5m','5min','5minutes']:
             end = self.clock.bars_start + self.clock.pre_bar*288
             start =end-bars
        k_lines_history = self.cacher.get_daily_ohlcv(k_lines_key, start, end, attributes)
        if rtype == 'ndarray':
            k_lines_history = np.array(k_lines_history)
        elif rtype == 'dataframe':
            k_lines_history = k_lines_history
        elif rtype == 'list':

            k_lines_history = np.array(k_lines_history).tolist()
        return k_lines_history

    def get_price(self, symbol):
        """
        :param symbol: "BTC/USDT.binance"
        :return:  回测：上一bar close_price
        """
        k_lines_key = symbol.split('.')[1] + '-spot-' + symbol.split('.')[0].replace('/', '').lower()
        freq1=self.frequency
        if freq1 in ['d','1d','day','1day']:
            last_bar_index =self.clock.bars_start + self.clock.pre_bar  # date取日级数据
        elif freq1 in ['m','1m','min','1minute']:
              last_bar_index = self.clock.bars_start + self.clock.pre_bar*1440
        elif freq1 in ['5m','5min','5minutes']:
             last_bar_index = self.clock.bars_start + self.clock.pre_bar*288


        df = self.cacher.data[k_lines_key]
        last_bar_price =df.loc[last_bar_index,'close'] #df.iloc[last_bar_index,:]['close']#
        return last_bar_price

    def get_depth(self,symbol_exchange):
        exchange = symbol_exchange.split('.')[1]
        symbol = symbol_exchange.split('.')[0]
        k_lines_key = exchange + '-spot-' + symbol.replace('/', '').lower()
        # date = self.clock.current_date
        freq1=self.frequency
        if freq1 in ['d','1d','day','1day']:
            last_bar_index =self.clock.bars_start + self.clock.pre_bar  # date取日级数据
        elif freq1 in ['m','1m','min','1minute']:
              last_bar_index = self.clock.bars_start + self.clock.pre_bar*1440
        elif freq1 in ['5m','5min','5minutes']:
             last_bar_index = self.clock.bars_start + self.clock.pre_bar*288

        df = self.cacher.data[k_lines_key]
        last_bar_price = df.loc[last_bar_index,'close'] #df.iloc[last_bar_index,:]['close'] 
        last_bar_qty = df.loc[last_bar_index,'volume'] #df.iloc[last_bar_index,:]['volume']# 
        if symbol=='btcusdt':
           tick1=0.5
        elif symbol=='ethusdt':
             tick1=0.05
        elif symbol=='eosusdt':
             tick1=0.001
        elif symbol=='xprusdt':
             tick1=0.0001  
        else:
             tick1=last_bar_price/10000
        depth={}
        ask1=(last_bar_price+tick1,last_bar_qty/10)
        ask2=(last_bar_price+2*tick1,last_bar_qty/10)
        ask3=(last_bar_price+3*tick1,last_bar_qty/10)
        ask4=(last_bar_price+4*tick1,last_bar_qty/10)
        ask5=(last_bar_price+5*tick1,last_bar_qty/10)
        bid1=(last_bar_price-tick1,last_bar_qty/10)
        bid2=(last_bar_price-2*tick1,last_bar_qty/10)
        bid3=(last_bar_price-3*tick1,last_bar_qty/10)
        bid4=(last_bar_price-4*tick1,last_bar_qty/10)
        bid5=(last_bar_price-5*tick1,last_bar_qty/10)
        depth['asks']=[ask1,ask2,ask3,ask4,ask5]
        depth['bids']=[bid1,bid2,bid3,bid4,bid5]

        return depth
        



'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
from vitu.account.account import Account

class SpotAccount(Account):
    def __init__(self, name, exchange, asset_class, current_position):
        """
        {"account_id": 1,
        "exchange": 'huobi',
        "api_key": 'spot_account',
        "market": 'spot',
        "position": ({'name': 'BTC', 'qty': 100}, {'name': 'USDT', 'qty': 200000})}
        """
        super(SpotAccount,self).__init__(name, exchange, asset_class, current_position)


'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
from vitu.data.data import get_data
from datetime import datetime

def get_ohlcv_data(universe, all_assets, benchmark, frequency, pre_start_date, end_date,timezone):
    ohlcv_data = dict()
    if frequency in ['d','1d','day','daily']: 
        freq = '1d'
    elif frequency in ['m','1m','min','minute']:
        freq ='1m'
    elif frequency in ['5m','5min','5minutes']:
        freq ='5m'
    for univ in universe:
        exchange = univ.split('.')[1]  # binance
        symbol = univ.split('.')[0].replace('/', '').lower()  # btcusdt
        symbol_cmc = symbol.replace('usdt', 'usd')
        k_lines_key = exchange + '-spot-' + symbol
        ohlcv_data[k_lines_key] = get_data(exchange, symbol, freq, pre_start_date, end_date,timezone)
        cmc_key = 'cmc-spot-' + symbol.replace('usdt', 'usd')
        #加入freq1 cmc和chainext只取天
        if freq in ['1m','5m']:
            freq1='1d'
        else: 
            freq1=freq    
        ohlcv_data[cmc_key] = get_data('cmc', symbol_cmc, freq1, pre_start_date, end_date,timezone)
    ohlcv_data['cmc-spot-usdtusd'] = get_data('cmc', 'usdtusd', freq1, pre_start_date, end_date,timezone)
    if benchmark == 'btc':
        ohlcv_data['chainext-index-' + benchmark] = get_data('cmc', 'btcusd', freq1, pre_start_date, end_date,timezone)
    else:
        st1=datetime.strptime(pre_start_date, "%Y-%m-%d %H:%M:%S")
        styear=st1.year
        if styear>2017:
           ohlcv_data['chainext-index-' + benchmark] = get_data('chainext', benchmark, freq1, pre_start_date, end_date,timezone)
        else:
            ohlcv_data['chainext-index-' + benchmark] = get_data('cmc','btcusd', freq1, pre_start_date, end_date,timezone)
    # 添加除universe里asset的cmc数据
    for asset in all_assets:
        symbol = '{}usd'.format(asset)
        cmc_key = 'cmc-spot-{}usd'.format(asset)
        if cmc_key not in ohlcv_data.keys():
            ohlcv_data[cmc_key] = get_data('cmc', symbol, freq1, pre_start_date, end_date,timezone)
    return ohlcv_data
#要加入分钟m,5m的取值
def get_btc_usdt_cost(asset, date):
    if asset == 'btc':
        avg_cost_btc = 1
        temp_getprice=get_data('binance', 'btcusdt', '1d', date,date,1)  # 默认都以binance为主
        avg_cost_usdt =temp_getprice.close[0]
        # avg_cost_usdt = get_price('binance', 'btcusdt', '1d', date)  # 默认都以binance为主
    elif asset == 'usdt':
        temp_getprice=get_data('binance', 'btcusdt', '1d', date,date,1)  # 默认都以binance为主
        avg_cost_btc=1/temp_getprice.close[0]
        # avg_cost_btc = 1 / get_price('binance', 'btcusdt', '1d', date)
        avg_cost_usdt = 1
    else:
        temp_getprice1=get_data('binance', '{}btc'.format(asset), '1d', date,date,1) 
        temp_getprice2=get_data('binance', '{}usdt'.format(asset), '1d', date,date,1) 
        avg_cost_btc=temp_getprice1.close[0]
        avg_cost_usdt=temp_getprice2.close[0]
        # avg_cost_btc = get_price('binance', '{}btc'.format(asset), '1d', date)
        # avg_cost_usdt = get_price('binance', '{}usdt'.format(asset), '1d', date)
    return avg_cost_btc,avg_cost_usdt









'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
class Universe(object):
    def get(self, query=None):
        pass

class StaticUniverse(Universe):
    def __init__(self, universe=None):
        if universe:
            self.universe = universe
        else:
            self.universe = list()

    def get(self, query=None):
        return self.universe


class IndexUniverse(Universe):
    def __init__(self, index):
        self.index = index

    def get(self, query=None):
        pass


if __name__ == '__main__':
    # universe.get(query={"filter_type": "volume", "order": "desc", "limit": "5"})
    u1 = StaticUniverse(['BTC/USDT.binance'])
    universe = u1.get()
    print(universe)


'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
from vitu.configuration import Config
from IPython.display import display
from pprint import pprint
import numpy as np
import pandas as pd
pd.set_option('display.max_rows',None)
pd.set_option('display.max_columns',None)
np.set_printoptions(threshold=10000000)
 

def output(report):
    if Config.mode() == 'jupyter':
        display(report)

    else:
        if isinstance(report,dict):
            for key,value in report.items():
                print(key,value)
        else:
            print(report)


'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
import copy
import datetime
import math

import numpy as np
from vitu.utils.date_utils import str2datetime, str2timestamp, get_dates_length

class Report(object):
    def __init__(self, portfolio=None):
        self.rebalance_history = portfolio.rebalance_history
        self.index_key = 'chainext-index-' + portfolio.strategy.benchmark
        
        self.context = portfolio.context
        self.refresh_rate = portfolio.strategy.refresh_rate

        self.dates = [date for date in self.rebalance_history.keys()]
        benchmark1= portfolio.context.cacher.data[self.index_key]
        self.benchmark=benchmark1[(benchmark1['timestamp']>=self.dates[0])&(benchmark1['timestamp']<=self.dates[-1])]
        self.dates_length = get_dates_length(self.dates[0], self.dates[-1])  # 计算年化时，算date_length,只算日长度
        # self.dates = None
        self.values = None
        self.bm_values = None

        self.annualized_return = None
        self.benchmark_annualized_return = None
        self.alpha = None
        self.sharpe = None
        self.information_ratio = None
        self.winning_ratio = None
        self.beta = None
        self.annualized_volatility = None
        self.max_drawdown = None

    def get_benchmark_values(self, dates):
        # 不管几点，要对应整点的的日级数据
        # start = str_to_timestamp(str((str_to_datetime(dates[0]) - datetime.timedelta(days=self.refresh_rate)).date()))
        if self.context.frequency in ['d','1d','day','1day']:
            start = str((str2datetime(dates[0]) - datetime.timedelta(days=self.refresh_rate)))  
            end = str((str2datetime(dates[-1])))
            bm_values = self.benchmark.loc[(self.benchmark["timestamp"] >= start) & (self.benchmark["timestamp"]<= end)]
            bm_values = bm_values.loc[bm_values.index[::self.refresh_rate]]['close'].tolist()
        else:
            start = str((str2datetime(dates[0]) - datetime.timedelta(days=1)))  #benchmarkvalue只取对应基准的日价值，所以每日值都取
            end = str((str2datetime(dates[-1])))
            bm_values = self.benchmark.loc[(self.benchmark["timestamp"] >= start) & (self.benchmark["timestamp"]<= end)]
            bm_values = bm_values['close'].tolist()
        return bm_values

    def get_relative_returns(self, values):
        """
        根据 总净值 计算 相对收益率
        :param values: 总净值 列表
        :return: 相对收益率 列表
        """
        returns = list()
        for num, value in enumerate(values[1:]):
            if not values[num] or not value or np.isnan(value / values[num]):
                returns.append(0)
                continue
            returns.append(round((value / values[num] - 1),4))
        return returns

    def get_log_returns(self, values):
        """
        :param values: 总净值 列表
        :return: 对数收益率 列表
        """
        # print(values)
        log_returns = []
        for i in range(len(values)-1):
            try:
                num = round(math.log(values[i + 1] / values[i],math.e), 4)
            except:
                num = 0
            log_returns.append(num)
        return log_returns
        # return [round(math.log(values[i + 1] / values[i],math.e), 4) for i in range(len(values) - 1)]

    def get_returns(self, values):
        """
        :param values: 总净值 列表
        :return: 累计收益率 列表
        """
        # print(values)
        return [round(values[i + 1] / values[i]-1, 4) for i in range(len(values) - 1)]

    # -----------------------------------------------
    def get_cumulative_returns(self, returns):
        """
        根据 相对收益率 计算 绝对收益率、收益率
        :param returns: 相对收益率 列表
        :return: 绝对收益率、收益率 列表
        """
        c_returns = copy.deepcopy(returns)
        try:
            c_returns[0] += 1
        except:
            return list()
        for i, r in enumerate(c_returns[1:]):
            c_returns[i + 1] = round(((1 + r) * c_returns[i]),4)
            if np.isnan(c_returns[i + 1]) or c_returns[i + 1] < 0:
                c_returns[i + 1] = 0
        c_returns_rate = [round(i-1,4) for i in c_returns]
        return c_returns,c_returns_rate

    def get_annualized_return(self, c_returns):
        """
        根据 累计收益率 计算 年化收益率
        :param c_returns: 累计收益率 列表
        :return: 年化收益率
        """
        return round((c_returns[-1] ** (365. / self.dates_length) - 1),4)
        # return round((c_returns[-1] ** (365. / len(c_returns)) - 1),4)


    def get_max_drawdown(self, c_returns):
        """
        根据 累计收益率 计算 最大回撤
        :param c_returns: 累计收益率 列表
        :return: 最大回撤
        """
        drawdown = []
        for i, v in enumerate(c_returns):
            j = max(c_returns[:i + 1])
            if j == 0:
                drawdown.append(0)
            else:
                drawdown.append(1 - v / j)
        # drawdown = [1 - v / max(c_returns[:i + 1]) for i, v in enumerate(c_returns)]
        return round((max(drawdown)),4)

    def get_riskfree_rate(self):
        """
        获得无风险利率
        """
        return 0.01

    def get_annualized_volatility(self, returns):
        """
        根据 相对收益率 计算 收益波动率 = 收益率的(年化)标准差
        :param returns: 相对收益率 列表
        :return: 收益波动率
        """
        # annualized_volatility = np.sqrt((365/(len(returns)-1))*sum([(i - np.mean(returns)) ** 2 for i in returns]))
        std_i = []
        for i in returns:
            std_i.append((i - np.mean(returns)) ** 2)
        annualized_volatility = np.sqrt((365/(len(returns)))*sum(std_i))
        # annualized_volatility = np.sqrt((365/(len(returns)))*sum([(i - np.mean(returns)) ** 2 for i in returns]))
        return round(annualized_volatility, 4)

    def get_beta(self, st_returns, bm_returns):
        """
        根据 策略累计收益率 基准累计收益率 计算 beta
        :param st_returns: 策略累计收益率 列表
        :param bm_returns: 基准累计收益率 列表
        :param rf: 无风险利率
        :return: beta：反映策略表现对大盘变化的敏感性，即策略与大盘的相关性
        """
        d = len(st_returns)
        if d == 1:
            # return None, None
            return 0, 0
        mean_st = sum(st_returns) / d
        mean_bm = sum(bm_returns) / d
        mul = [st_returns[i] * bm_returns[i] for i in range(d)]
        cov = sum(mul) / d - mean_st * mean_bm
        var_bm = np.std(bm_returns) ** 2
        if var_bm == 0:
            return 0
        beta = cov / var_bm
        return round(beta,4)

    def get_alpha(self, st_returns, bm_returns, rf):
        """
        根据 策略累计收益率 基准累计收益率 无风险利率 计算 alpha
        :param st_returns: 策略累计收益率 列表
        :param bm_returns: 基准累计收益率 列表
        :param rf: 无风险利率
        :return: alpha：阿尔法是超额收益，它与市场波动无关
        """
        d = len(st_returns)
        if d == 1:
            # return None, None
            return 0, 0
        mu_st = sum(st_returns) / d
        mu_bm = sum(bm_returns) / d
        mul = [st_returns[i] * bm_returns[i] for i in range(d)]
        cov = sum(mul) / d - mu_st * mu_bm
        var_bm = np.std(bm_returns) ** 2
        if np.isnan(cov / var_bm):
            # return None, None
            return 0, 0
        beta = cov / var_bm
        if beta in [-np.inf, np.inf] or np.isnan(beta):
            return 0, 0
            # return None, None
        cummulative_st = self.get_cumulative_returns(st_returns)[0]
        annulized_st = self.get_annualized_return(cummulative_st)
        cummulative_bm = self.get_cumulative_returns(bm_returns)[0]
        annulized_bm = self.get_annualized_return(cummulative_bm)
        alpha = (annulized_st - rf) - beta * (annulized_bm - rf)
        return round(alpha,4)

    def get_CAPM(self, st_returns, bm_returns, rf):
        """
        根据 策略累计收益率 基准累计收益率 无风险利率 计算 beta和alpha
        :param st_returns: 策略累计收益率 列表
        :param bm_returns: 基准累计收益率 列表
        :param rf: 无风险利率
        :return: beta和alpha
        """
        d = len(st_returns)
        if d == 1:
            # return None, None
            return 0, 0
        mu_st = sum(st_returns) / d
        mu_bm = sum(bm_returns) / d
        mul = [st_returns[i] * bm_returns[i] for i in range(d)]

        cov = sum(mul) / d - mu_st * mu_bm
        var_bm = np.std(bm_returns) ** 2
        if not var_bm:
            # return None,None
            return 0, 0
        beta = cov / var_bm
        if beta in [-np.inf, np.inf] or np.isnan(beta):
            # return None, None
            return 0, 0
        cummulative_st = self.get_cumulative_returns(st_returns)[0]
        annulized_st = self.get_annualized_return(cummulative_st)
        cummulative_bm = self.get_cumulative_returns(bm_returns)[0]
        annulized_bm = self.get_annualized_return(cummulative_bm)

        alpha = (annulized_st - rf) - beta * (annulized_bm - rf)
        return round(alpha,4), round(beta,4)

    def get_sharpe(self, st_returns, rf):
        """
        根据 策略累计收益率 无风险利率 计算 sharpe
        :param st_returns: 策略累计收益率 列表
        :param rf: 无风险利率
        :return: sharpe：单位总风险下所能获得的超额收益，即每承受一单位总风险，会产生多少的超额收益
        """
        cummulative_st = self.get_cumulative_returns(st_returns)[0]
        annulized_st = self.get_annualized_return(cummulative_st)
        volatility_st = self.get_annualized_volatility(st_returns)
        if volatility_st == 0:
            return 0
        sharpe = (annulized_st - rf) / volatility_st
        return round(sharpe,4)

    def get_information_ratio(self, st_returns, bm_returns):
        """
        根据 策略累计收益率 基准累计收益率 计算 信息比率IR
        :param st_returns: 策略累计收益率 列表
        :param bm_returns: 基准累计收益率 列表
        :return: IR：单位超额风险下的超额收益。它用于衡量单位超额风险带来的超额收益。信息比率越大，说明该策略单位跟踪误差所获得的超额收益越高
        """
        d = len(st_returns)
        if d == 1:
            return 0
        diff = [st_returns[i] - bm_returns[i] for i in range(d)]
        cummulative_st = self.get_cumulative_returns(st_returns)[0]
        annulized_st = self.get_annualized_return(cummulative_st)
        cummulative_bm = self.get_cumulative_returns(bm_returns)[0]
        annulized_bm = self.get_annualized_return(cummulative_bm)
        annulized_std = (np.var(diff, ddof=1) * 365) ** 0.5
        if annulized_std == 0:
            return 0
        IR = (annulized_st - annulized_bm) / annulized_std
        if IR in [np.inf, -np.inf] or np.isnan(IR):
            return 0
        return round(IR,4)

    def __getitem__(self, key, default=None):
        item_value = self.__getattribute__(key) if self.__getattribute__(key) else default
        return item_value

if __name__ == '__main__':
    print(round(((1.26 ** (365. / 690)) - 1), 4))
    print(round(((1.3 ** (365. / 720)) - 1), 4))


'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''

import time
import asyncio

from vitu.account.account_manager import AccountManager
from vitu.strategy.strategy import Strategy
from vitu.universe.universe import StaticUniverse, IndexUniverse
from vitu.configuration import Config
from vitu.utils.output_utils import output

def create_account(name=None, exchange=None, account_type=None, position_base=None):
    """
    :param name: 'spot_account'
    :param exchange: 'binance'
    :param account_type: 'digital.spot'
    :param position: ({'asset': 'BTC', 'qty': 100}, {'asset': 'USDT', 'qty': 200000}))
    :return: AccountManager instance
    """
    AccountManager().config(name, exchange, account_type, position_base)


def create_strategy(initialize=None, handle_data=None, universe=None, benchmark=None,
                    freq=None, refresh_rate=None):
    """
    :param initialize: initialize
    :param handle_data: handle_data
    :param universe: ['BTC/USDT.okex', 'ETH/BTC.okex', 'XBT/USD.bitmex']
    :param benchmark: 'csi5'/'btc'
    :param freq: 'd'
    :param refresh_rate: 1 or (1,['08:00:00']) or (1,['08:00:00','18:00:00'])
    :return: strategy_instance
    """
    return Strategy(initialize, handle_data, universe, benchmark, freq, refresh_rate)


def create_universe(universe=None):
    """
    :param universe: ['BTC/USDT.okex', 'ETH/BTC.okex']/'csi5'
    :return:
    """
    if universe:
        if universe in Config.benchmarks():
            return IndexUniverse(universe).get()
        else:
            return StaticUniverse(universe).get()

error_complete = 0

# async def _heartbeat(strategy):
#     global error_complete
#     try:
#         while True:
            
#             clock = strategy.portfolio.context.clock
#             timenow=time.time()
#             if (timenow-clock.datestart)> Config.heartbeat_timelength():
#                 clock.reset_datestart()
#                 output('heartbeating')
#             await asyncio.sleep(0)
#             if error_complete == 1:
#                 return
#             if clock.current_timestamp > clock.end_timestamp:
#                 break
#     except Exception as e:
#         error_complete = 1
#         raise e
async def _handle(strategy):
    global error_complete
    try:
        while True:
            clock = strategy.portfolio.context.clock
            timenow=time.time()
            if (timenow-clock.datestart)> Config.heartbeat_timelength():
                clock.reset_datestart()
                output('heartbeating')
            if error_complete == 1:
                return
            if clock.current_timestamp > clock.end_timestamp:
                break
            error_complete = await strategy._handle_data()
            clock.next()
            await asyncio.sleep(0)
    except Exception as e:
        error_complete = 1
        raise e

async def _report(strategy):
    global error_complete
    try:
        while True:
            await asyncio.sleep(1)
            if error_complete == 1:
                return
            clock = strategy.portfolio.context.clock
            if clock.current_timestamp > clock.end_timestamp:
                clock.run_end = time.time()
                strategy.portfolio.context.completed_time = round(clock.run_end - clock.run_start,4)
                # print('【handle总耗时】：{} s'.format(strategy.portfolio.context.completed_time))
                # strategy.simple_report()
                #strategy.complete_report()
                strategy.simplereport_excel()
                break


            if strategy.frequency in ['d','1d','day','1day'] :
               if clock.bars > Config.bars_length():  
                  clock.reset_bars()
                  strategy.simple_report()
            elif strategy.frequency in ['m','1m','min','minute'] :
               if clock.bars >Config.bars_length_1m():  
                  clock.reset_bars()
                  strategy.simple_report()


            elif strategy.frequency in ['5m','5mins','5minutes'] :
               if clock.bars >Config.bars_length_5m():  
                  clock.reset_bars()
                  strategy.simple_report()
         
    except Exception as e:
        error_complete = 1
        raise e


def backtest(strategy, start, end, commission, timezone=1):
    """
    :param strategy: strategy
    :param start: '2019-06-01'  # 回测起始时间
    :param end: '2019-07-01'    # 回测结束时间
    :param commission: {'buy': 0.001, 'sell': 0.001, 'open': 0.001, 'close': 0.001}
    :return: portfolio
    :timezone:timezone
    """
    global error_complete
    error_complete = 0
    strategy._initialize(strategy, start, end, timezone, commission)
    loop = asyncio.get_event_loop()
    if Config.mode() == 'jupyter':
        loop.create_task(_handle(strategy))
        loop.create_task(_report(strategy))
    else:
        loop.run_until_complete(asyncio.gather(_handle(strategy), _report(strategy))) #_heartbeat(strategy),
        loop.close()






'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
from vitu.utils.date_utils import str2timestamp

class DataCache(object):
    def __init__(self):
        self.data = dict()

    def get_daily_ohlcv(self, k_lines_key, start, end, attributes):
        """
        :param k_lines_key: 'spot-binance-btcusdt'
        :param start_date: datetime
        :param end_date: datetime
        :return: dataframe
        """
        # start_ts = str(start)
        # end_ts = str(end)
        total_data = self.data[k_lines_key]
        data = total_data.loc[start:end-1,attributes] #total_data.iloc[start:end,:][attributes] 
        return data





'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
class PortfolioPosition():
    def __init__(self, context=None, asset=None, accounts=None):
        self.context = context
        self.asset = asset
        self.accounts = accounts
          
        self.asset_class = None
        self.total_amount = 0
        self.value = 0
        self.total_qty = 0

        self.avg_cost_btc_total = 0
        self.avg_cost_usdt_total = 0
        self.avg_cost_btc = 0
        self.avg_cost_usdt = 0

        self.consis_of = dict()

    def detail(self):
        for account in self.accounts.values():
            account = account.to_dict()
            if self.asset in account['current_position'].keys():
                self.asset_class = account['asset_class']
                temp1=account['current_position'][self.asset].detail()
                self.consis_of[account['name']] =temp1 
                self.total_amount += temp1['amount']
                self.value = temp1['value']
                self.total_qty += temp1['total']
                self.avg_cost_btc_total += temp1['avg_cost_btc']
                self.avg_cost_usdt_total += temp1['avg_cost_usdt']
                
        self.avg_cost_btc = self.avg_cost_btc_total/len(self.consis_of)
        self.avg_cost_usdt = self.avg_cost_usdt_total/len(self.consis_of)

        return {
            'asset':self.asset,
            'asset_class' : self.asset_class,
            'total_amount': round(self.total_amount, 4),
            'value': round(self.value,4),
            'total_qty': round(self.total_qty, 4),
            'avg_cost_btc': self.avg_cost_btc,
            'avg_cost_usdt': self.avg_cost_usdt,
            'rebalance_time':self.context.current_datetime(),
            'consist_of': self.consis_of  #  key = accountName, value = position
        }

    def __getitem__(self, key, default=None):
        item_value = self.__getattribute__(key) if self.__getattribute__(key) else default
        return item_value

if __name__ == '__main__':
    pass



'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
#!/usr/bin/env python
#coding:utf-8

'''
取数据相关接口
'''
import sys

import pytz

from vitudata.exceptions import ParamsError

if sys.version_info[0] == 3:
    xrange = range
    
import warnings
import datetime
import collections
import os
import re
import numpy as np


def is_str(s):
    return isinstance(s, six.string_types)

def is_list(l):
    return isinstance(l, (list, tuple))


def is_empty(container):
    """ 检查容器类数据不为空 dict, set, list, queue ..."""
    return len(container) == 0


import six


def check_string(s):
    assert is_str(s), "参数必须是个字符串, 实际是:%s" % str(type(s))


def check_list(l):
    assert isinstance(l, (tuple, list)), "参数必须是tuple或者list, 实际是: %s" % str(type(l))


def ensure_str_tuple(args):
    if is_str(args):
        return (args,)
    else:
        atuple = tuple(args)
        for i in atuple:
            assert isinstance(i, six.string_types)
        return atuple


def date2dt(date):
    return datetime.datetime.combine(date, datetime.time.min)


def convert_dt(dt):
    if is_str(dt):
        if ':' in dt:
            return datetime.datetime.strptime(dt, '%Y-%m-%d %H:%M:%S').replace(tzinfo=pytz.timezone('utc'))
        else:
            return datetime.datetime.strptime(dt, '%Y-%m-%d').replace(tzinfo=pytz.timezone('utc'))
    elif isinstance(dt, datetime.datetime):
        if dt.tzinfo is None:
            dt.replace(tzinfo=pytz.timezone('utc'))
        return dt
    elif isinstance(dt, datetime.date):
        if dt.tzinfo is None:
            dt.replace(tzinfo=pytz.timezone('utc'))
        return date2dt(dt)
    raise ParamsError("date 必须是datetime.date, datetime.datetime或者如下格式的字符串:'2018-10-05'")


def convert_date(date):
    if is_str(date):
        if ':' in date:
            date = date[:10]
        return datetime.datetime.strptime(date, '%Y-%m-%d').date()
    elif isinstance(date, datetime.datetime):
        return date.date()
    elif isinstance(date, datetime.date):
        return date
    raise ParamsError("date 必须是datetime.date, datetime.datetime或者如下格式的字符串:'2018-10-05'")


def all_is_none(*args):
    for obj in args:
        if obj is not None:
            return False
    return True


def all_not_none(*args):
    for obj in args:
        if obj is None:
            return False
    return True
